//+------------------------------------------------------------------+
//|                                          Global Market Trend MTF |
//|                      Copyright 2013, Deltabron - Paul Geirnaerdt |
//|                                          http://www.deltabron.nl |
//+------------------------------------------------------------------+
#property copyright "Copyright 2013, Deltabron - Paul Geirnaerdt"
#property link      "http://www.deltabron.nl"

#property indicator_separate_window
#property indicator_buffers 8

#define version            "v1.0.0"

#define EPSILON            0.00000001

#define CURRENCYCOUNT      8

//+------------------------------------------------------------------+
//| Release Notes                                                    |
//+------------------------------------------------------------------+
// v1.0.0, 7/29/13
// * Initial release

extern string  gen               = "----General inputs----";
extern string  nonPropFont       = "Lucida Console";
extern int     fontSize          = 14;

extern string  ind               = "----Indicator inputs----";
extern bool    autoSymbols       = false;
extern string  symbolsToWeigh    = "GBPNZD,EURNZD,GBPAUD,GBPCAD,GBPJPY,GBPCHF,CADJPY,EURCAD,EURAUD,USDCHF,GBPUSD,EURJPY,NZDJPY,AUDCHF,AUDJPY,USDJPY,EURUSD,NZDCHF,CADCHF,AUDNZD,NZDUSD,CHFJPY,AUDCAD,USDCAD,NZDCAD,AUDUSD,EURCHF,EURGBP";
extern bool    ignoreFuture      = true;
extern int     maxBars           = 500;
extern int     lineWidth         = 2;
extern bool    showW1            = false;
extern bool    showD1            = true;
extern bool    showH4            = true;
extern bool    showH1            = false;
extern bool    showM30           = false;
extern bool    showM15           = false;
extern bool    showM5            = false;
extern bool    showM1            = false;
extern string  ind_sm            = "----Add Sunday candles to Monday?----";
extern bool    addSundayToMonday = true;

extern string  col               = "----Colo(u)rs----";
extern color   colorW1           = C'0xA2,0x48,0x92';
extern color   colorD1           = C'0xBA,0x55,0xDF';
extern color   colorH4           = C'0x6F,0x59,0x61';
extern color   colorH1           = C'0xD8,0xDA,0x8E';
extern color   colorM30          = C'0xA2,0x98,0x30';
extern color   colorM15          = C'0x5F,0x88,0x1B';
extern color   colorM5           = C'0xA4,0xD5,0x55';
extern color   colorM1           = C'0xE2,0x4F,0x88';

//---- buffers
double W1Buffer[];
double D1Buffer[];
double H4Buffer[];
double H1Buffer[];
double M30Buffer[];
double M15Buffer[];
double M5Buffer[];
double M1Buffer[];

//---- global variables
string   indicatorName = "Global Market Trend MTF";
string   shortName;
datetime currentTime;
bool     sundayCandlesDetected;                          
int      userTimeFrame;

//---- symbol & currency variables
int      symbolCount;
string   symbolNames[];
string   currencyNames[CURRENCYCOUNT]        = { "USD", "EUR", "GBP", "CHF", "JPY", "AUD", "CAD", "NZD" };
double   currencyValues[CURRENCYCOUNT];      // Currency slope strength
double   currencyOccurrences[CURRENCYCOUNT]; // Holds the number of occurrences of each currency in symbols

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   initSymbols();
 
   shortName = indicatorName + " - " + version;
   IndicatorShortName(shortName);

   SetIndexBuffer ( 0, W1Buffer );
   SetIndexBuffer ( 1, D1Buffer );
   SetIndexBuffer ( 2, H4Buffer );
   SetIndexBuffer ( 3, H1Buffer );
   SetIndexBuffer ( 4, M30Buffer );
   SetIndexBuffer ( 5, M15Buffer );
   SetIndexBuffer ( 6, M5Buffer );
   SetIndexBuffer ( 7, M1Buffer );

   SetIndexLabel ( 0, "W1" );
   SetIndexLabel ( 1, "D1" );
   SetIndexLabel ( 2, "H4" );
   SetIndexLabel ( 3, "H1" );
   SetIndexLabel ( 4, "M30" );
   SetIndexLabel ( 5, "M15" );
   SetIndexLabel ( 6, "M5" );
   SetIndexLabel ( 7, "M1" );
   
   sundayCandlesDetected = false;
   for ( int i = 0; i < 8; i++ )
   {
      if ( TimeDayOfWeek( iTime( NULL, PERIOD_D1, i ) ) == 0 )
      {
         sundayCandlesDetected = true;
         break;
      }
   }
   
   return ( 0 );
}

//+------------------------------------------------------------------+
//| Initialize Symbols Array                                         |
//+------------------------------------------------------------------+
void initSymbols()
{
   int i;
   
   // Get extra characters on this crimmal's symbol names
   string symbolExtraChars = StringSubstr(Symbol(), 6, 4);

   // Trim user input
   symbolsToWeigh = StringTrimLeft(symbolsToWeigh);
   symbolsToWeigh = StringTrimRight(symbolsToWeigh);

   // Add extra comma
   if (StringSubstr(symbolsToWeigh, StringLen(symbolsToWeigh) - 1) != ",")
   {
      symbolsToWeigh = StringConcatenate(symbolsToWeigh, ",");   
   }   

   // Build symbolNames array as the user likes it
   if ( autoSymbols )
   {
      createSymbolNamesArray();
   }
   else
   {
      // Split user input
      i = StringFind(symbolsToWeigh, ","); 
      while (i != -1)
      {
         int size = ArraySize(symbolNames);
         string newSymbol = StringConcatenate(StringSubstr(symbolsToWeigh, 0, i), symbolExtraChars);
         if ( iClose(newSymbol,0,0) != 0)
         {
            ArrayResize( symbolNames, size + 1 );
            // Set array
            symbolNames[size] = newSymbol;
         }
         // Trim symbols
         symbolsToWeigh = StringSubstr(symbolsToWeigh, i + 1);
         i = StringFind(symbolsToWeigh, ","); 
      }
   }
   
   symbolCount = ArraySize(symbolNames);

   for ( i = 0; i < symbolCount; i++ )
   {
      // Increase currency occurrence
      int currencyIndex = GetCurrencyIndex(StringSubstr(symbolNames[i], 0, 3));
      currencyOccurrences[currencyIndex]++;
      currencyIndex = GetCurrencyIndex(StringSubstr(symbolNames[i], 3, 3));
      currencyOccurrences[currencyIndex]++;
   }   

}

//+------------------------------------------------------------------+
//| GetCurrencyIndex(string currency)                                |
//+------------------------------------------------------------------+
int GetCurrencyIndex(string currency)
{
   for (int i = 0; i < CURRENCYCOUNT; i++)
   {
      if (currencyNames[i] == currency)
      {
         return(i);
      }   
   }   
   return (-1);
}

//+------------------------------------------------------------------+
//| createSymbolNamesArray()                                         |
//+------------------------------------------------------------------+
void createSymbolNamesArray()
{
   int hFileName = FileOpenHistory ("symbols.raw", FILE_BIN|FILE_READ );
   int recordCount = FileSize ( hFileName ) / 1936;
   int counter = 0;
   for ( int i = 0; i < recordCount; i++ )
   {
      string tempSymbol = StringTrimLeft ( StringTrimRight ( FileReadString ( hFileName, 12 ) ) );
      if ( MarketInfo ( tempSymbol, MODE_BID ) > 0 && MarketInfo ( tempSymbol, MODE_TRADEALLOWED ) )
      {
         ArrayResize( symbolNames, counter + 1 );
         symbolNames[counter] = tempSymbol;
         counter++;
      }
      FileSeek( hFileName, 1924, SEEK_CUR );
   }
   FileClose( hFileName );
}

//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   int windex = WindowFind ( shortName );
   if ( windex > 0 )
   {
      ObjectsDeleteAll ( windex );
   }   

   return ( 0 );
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   SetIndexStyle ( 0, DRAW_LINE, STYLE_SOLID, lineWidth, colorW1 );
   SetIndexStyle ( 1, DRAW_LINE, STYLE_SOLID, lineWidth, colorD1 );
   SetIndexStyle ( 2, DRAW_LINE, STYLE_SOLID, lineWidth, colorH4 );
   SetIndexStyle ( 3, DRAW_LINE, STYLE_SOLID, lineWidth, colorH1 );
   SetIndexStyle ( 4, DRAW_LINE, STYLE_SOLID, lineWidth, colorM30 );
   SetIndexStyle ( 5, DRAW_LINE, STYLE_SOLID, lineWidth, colorM15 );
   SetIndexStyle ( 6, DRAW_LINE, STYLE_SOLID, lineWidth, colorM5 );
   SetIndexStyle ( 7, DRAW_LINE, STYLE_SOLID, lineWidth, colorM1 );

   int i;
   int counted_bars = IndicatorCounted();

   if ( counted_bars < 0 ) return ( -1 );
   if ( counted_bars > 0 ) counted_bars--;
   int limit = Bars - counted_bars;
   if ( maxBars > 0 && limit > maxBars ) limit = maxBars;

   for ( i = 0; i < limit; i++ )
   {
      currentTime = Time[i];
      
      if ( showW1 ) W1Buffer[i] = calcGlobalMarketTrend( PERIOD_W1, iBarShift ( NULL, PERIOD_W1, currentTime ) );
      if ( showD1 ) D1Buffer[i] = calcGlobalMarketTrend( PERIOD_D1, iBarShift ( NULL, PERIOD_D1, currentTime ) );
      if ( showH4 ) H4Buffer[i] = calcGlobalMarketTrend( PERIOD_H4, iBarShift ( NULL, PERIOD_H4, currentTime ) );
      if ( showH1 ) H1Buffer[i] = calcGlobalMarketTrend( PERIOD_H1, iBarShift ( NULL, PERIOD_H1, currentTime ) );
      if ( showM30 ) M30Buffer[i] = calcGlobalMarketTrend( PERIOD_M30, iBarShift ( NULL, PERIOD_M30, currentTime ) );
      if ( showM15 ) M15Buffer[i] = calcGlobalMarketTrend( PERIOD_M15, iBarShift ( NULL, PERIOD_M15, currentTime ) );
      if ( showM5 ) M5Buffer[i] = calcGlobalMarketTrend( PERIOD_M5, iBarShift ( NULL, PERIOD_M5, currentTime ) );
      if ( showM1 ) M1Buffer[i] = calcGlobalMarketTrend( PERIOD_M1, iBarShift ( NULL, PERIOD_M1, currentTime ) );
      
      if ( i == 0 )
      {
         int row = 0;
         if ( showW1 ) { ShowTable("W1", W1Buffer[i], colorW1, row); row++; }
         if ( showD1 ) { ShowTable("D1", D1Buffer[i], colorD1, row); row++; }
         if ( showH4 ) { ShowTable("H4", H4Buffer[i], colorH4, row); row++; }
         if ( showH1 ) { ShowTable("H1", H1Buffer[i], colorH1, row); row++; }
         if ( showM30 ) { ShowTable("M30", M30Buffer[i], colorM30, row); row++; }
         if ( showM15 ) { ShowTable("M15", M15Buffer[i], colorM15, row); row++; }
         if ( showM5 ) { ShowTable("M5", M5Buffer[i], colorM5, row); row++; }
         if ( showM1 ) { ShowTable("M1", M1Buffer[i], colorM1, row); row++; }
      }   
   }

   return ( 0 );
}

//+------------------------------------------------------------------+
//| calcGlobalMarketTrend( int tf, int shift )                       |
//+------------------------------------------------------------------+
double calcGlobalMarketTrend( int tf, int shift ) 
{
   ArrayInitialize(currencyValues, 0.0);
   calcCSS( tf, shift );

   double buffer = 0;
   for ( int i = 0; i < 8; i++ )
   {
      buffer += MathPow(currencyValues[i], 2);
   }
   return ( buffer );
}

//+------------------------------------------------------------------+
//| calcCSS( int tf, int shift )                                     |
//+------------------------------------------------------------------+
void calcCSS( int tf, int shift )
{
   int i;
   // Get Slope for all symbols and totalize for all currencies   
   for ( i = 0; i < symbolCount; i++)
   {
      double slope = GetSlope(symbolNames[i], tf, shift);
      currencyValues[GetCurrencyIndex(StringSubstr(symbolNames[i], 0, 3))] += slope;
      currencyValues[GetCurrencyIndex(StringSubstr(symbolNames[i], 3, 3))] -= slope;
   }
   for ( i = 0; i < CURRENCYCOUNT; i++ )
   {
      // average
      currencyValues[i] /= currencyOccurrences[i];
   }
}

//+------------------------------------------------------------------+
//| GetSlope()                                                       |
//+------------------------------------------------------------------+
double GetSlope(string symbol, int tf, int shift)
{
   double dblTma, dblPrev;
   int shiftWithoutSunday = shift;
   if ( addSundayToMonday && sundayCandlesDetected && tf == PERIOD_D1 )
   {
      if ( TimeDayOfWeek( iTime( symbol, PERIOD_D1, shift ) ) == 0  ) shiftWithoutSunday++;
   }   
   double atr = iATR(symbol, tf, 100, shiftWithoutSunday + 10) / 10;
   double gadblSlope = 0.0;
   if ( atr != 0 )
   {
      if ( ignoreFuture )
      {
         dblTma = calcTmaTrue( symbol, tf, shiftWithoutSunday );
         dblPrev = calcPrevTrue( symbol, tf, shiftWithoutSunday );
      }
      else
      {   
         dblTma = calcTma( symbol, tf, shiftWithoutSunday );
         dblPrev = calcTma( symbol, tf, shiftWithoutSunday + 1 );
      }   
      gadblSlope = ( dblTma - dblPrev ) / atr;
   }
   
   return ( gadblSlope );

}

//+------------------------------------------------------------------+
//| calcTma()                                                        |
//+------------------------------------------------------------------+
double calcTma( string symbol, int tf,  int shift )
{
   double dblSum  = iClose( symbol, tf, shift ) * 21;
   double dblSumw = 21;
   int jnx, knx;
         
   for ( jnx = 1, knx = 20; jnx <= 20; jnx++, knx-- )
   {
      dblSum  += iClose(symbol, tf, shift + jnx) * knx;
      dblSumw += knx;

      if ( jnx <= shift )
      {
         dblSum  += iClose(symbol, tf, shift - jnx) * knx;
         dblSumw += knx;
      }
   }
   
   return ( dblSum / dblSumw );
}


//+------------------------------------------------------------------+
//| calcTmaTrue()                                                    |
//+------------------------------------------------------------------+
double calcTmaTrue( string symbol, int tf, int inx )
{
   return ( iMA( symbol, tf, 21, 0, MODE_LWMA, PRICE_CLOSE, inx ) );
}

//+------------------------------------------------------------------+
//| calcPrevTrue()                                                   |
//+------------------------------------------------------------------+
double calcPrevTrue( string symbol, int tf, int inx )
{
   double dblSum  = iClose( symbol, tf, inx + 1 ) * 21;
   double dblSumw = 21;
   int jnx, knx;
   
   dblSum  += iClose( symbol, tf, inx ) * 20;
   dblSumw += 20;
         
   for ( jnx = 1, knx = 20; jnx <= 20; jnx++, knx-- )
   {
      dblSum  += iClose( symbol, tf, inx + 1 + jnx ) * knx;
      dblSumw += knx;
   }
   
   return ( dblSum / dblSumw );
}
 
//+------------------------------------------------------------------+
//| ShowTable()                                              |
//+------------------------------------------------------------------+
void ShowTable( string showText, double showValue, color showColor, int row )
{
   string objectName;
   int windex = WindowFind ( shortName );

   objectName = "gmtmtf_obj_name_" + row;
   if ( ObjectFind ( objectName ) == -1 )
   {
      if ( ObjectCreate ( objectName, OBJ_LABEL, windex, 0, 0 ) )
      {
         ObjectSet ( objectName, OBJPROP_CORNER, 1 );
         ObjectSet ( objectName, OBJPROP_XDISTANCE, 75 );
         ObjectSet ( objectName, OBJPROP_YDISTANCE, 20 * row + 10 );
      }
   }
   ObjectSetText ( objectName, showText, fontSize, nonPropFont, showColor );

   objectName = "gmtmtf_obj_value_" + row;
   if ( ObjectFind ( objectName ) == -1 )
   {
      if ( ObjectCreate ( objectName, OBJ_LABEL, windex, 0, 0 ) )
      {
         ObjectSet ( objectName, OBJPROP_CORNER, 1 );
         ObjectSet ( objectName, OBJPROP_XDISTANCE, 75 - 60 );
         ObjectSet ( objectName, OBJPROP_YDISTANCE, 20 * row + 10 );
      }
   }
   showText = RightAlign(DoubleToStr(showValue, 2), 5);
   ObjectSetText ( objectName, showText, fontSize, nonPropFont, showColor );
}
   
//+------------------------------------------------------------------+
//| Right Align Text                                                 |
//+------------------------------------------------------------------+
string RightAlign ( string text, int length = 10, int trailing_spaces = 0 )
{
   string text_aligned = text;
   for ( int i = 0; i < length - StringLen ( text ) - trailing_spaces; i++ )
   {
      text_aligned = " " + text_aligned;
   }
   return ( text_aligned );
}

//+------------------------------------------------------------------+

